#define use_cubic_filt $use_cubic_filt
#define use_shading 0
#define use_bg_ignore 0
#define use_segmentation_mask $use_segmentation_mask

#define global_opacity 1024.0

varying vec3 vertex,normal;	

uniform float min_level,max_level, step_length;
uniform vec3 pos;		
uniform vec3 nav;	
uniform vec3 cell_size;	
uniform vec3 cell_size_block;
uniform vec4 params;	
uniform float width,height;	
uniform vec3 box1,box2;
uniform vec3 txt_box1,txt_box2;
uniform vec4 random_seed;
uniform vec3 light_pos;

uniform sampler2D screen_text;
uniform sampler2D distance_map_text;
uniform sampler3D f_text;
uniform sampler3D f_text_i;		
uniform sampler2D transfer_function;

uniform vec3 RFrom;
uniform vec3 GFrom;
uniform vec3 BFrom;
uniform vec3 LightDir;
uniform vec3 gbox1,gbox2;



const vec3 level_color1 = vec3(0.6);	


const float z_near=0.3;
const float z_far=60.0;



vec4 GetLevelColor(float lv) 
{

//return vec4(max(0.0,sin(lv)));
	return texture2D(transfer_function, vec2(clamp((lv-min_level)/(max_level-min_level),0.0/512.0,512.0/512.0),0.5));
}
float interpolate_cubic(sampler3D tex, vec3 coord,vec3 cell_size1)
{
	
	// transform the coordinate from [0,extent] to [-0.5, extent-0.5]
	vec3 coord_grid = coord/cell_size1-vec3(0.5);
	vec3 index = floor(coord_grid);
	vec3 fraction = coord_grid - index;
	vec3 one_frac = vec3(1.0) - fraction;
	vec3 one_frac2 = one_frac * one_frac;
	vec3 fraction2 = fraction * fraction;

	vec3 w0 = 1.0/6.0 * one_frac2 * one_frac;
	vec3 w1 = vec3(2.0/3.0) - 0.5 * fraction2 * (2.0-fraction);
	vec3 w2 = vec3(2.0/3.0) - 0.5 * one_frac2 * (2.0-one_frac);
	vec3 w3 = 1.0/6.0 * fraction2 * fraction;
	vec3 g0 = w0 + w1;
	vec3 g1 = w2 + w3;
	// h0 = w1/g0 - 1, move from [-0.5, extent-0.5] to [0, extent]
	vec3 h0 = (w1 / g0) - vec3(0.5) + index;
	vec3 h1 = (w3 / g1) + vec3(1.5) + index;
	h0*=cell_size1;
	h1*=cell_size1;

	// fetch the four linear interpolations
	
	float tex000 = texture3D(tex, h0).x;
	float tex100 = texture3D(tex, vec3(h1.x, h0.y,h0.z)).x;
	float tex010 = texture3D(tex, vec3(h0.x, h1.y,h0.z)).x;
	float tex110 = texture3D(tex, vec3(h1.x,h1.y,h0.z)).x;
	
	float tex001 = texture3D(tex, vec3(h0.x,h0.y,h1.z)).x;
	float tex101 = texture3D(tex, vec3(h1.x,h0.y,h1.z)).x;
	float tex011 = texture3D(tex, vec3(h0.x,h1.y,h1.z)).x;
	float tex111 = texture3D(tex, h1).x;
	// weigh along the z-direction
	tex000 = mix(tex001, tex000, g0.z);
	tex100 = mix(tex101, tex100, g0.z);
	tex010 = mix(tex011, tex010, g0.z);
	tex110 = mix(tex111, tex110, g0.z);

	// weigh along the y-direction
	tex000 = mix(tex010, tex000, g0.y);
	tex100 = mix(tex110, tex100, g0.y);
	// weigh along the x-direction
	return mix(tex100, tex000, g0.x);
}

float Equ(vec3 arg)
{
	#if use_cubic_filt
		return interpolate_cubic(f_text,(arg-txt_box1)/(txt_box2-txt_box1),cell_size_block);
	#else
		return texture3D(f_text, (arg-txt_box1)/(txt_box2-txt_box1)).x;
	#endif
	
}

float Equ1(in vec3 arg)
{

	return texture3D(f_text, (arg-txt_box1)/(txt_box2-txt_box1)).x;
	
}

#if use_segmentation_mask
float Equ_i(in vec3 arg)
{

	return texture3D(f_text_i, (arg-gbox1)/(gbox2-gbox1)).x;
	
}
#endif


#define K_A 0.1
#define K_D 1.0
#define K_S 0.3
#define P 32.0

vec3 Phong ( vec3 point, vec3 normal, vec3 color)
{
//	vec3 LightDir = -normalize(point-light_pos);
	float diffuse = dot ( LightDir, normal )*0.5+0.5;
	vec3 reflect = reflect ( -LightDir, normal );
	float specular = pow ( max (0.0, dot ( reflect, normalize(pos-point) ) ), P );
	return K_A * color + diffuse * ( K_D * color + K_S * specular * vec3(1.0) );
}





bool IntersectBox ( vec3 Origin,vec3 Direction, vec3 minimum, vec3 maximum, out float start, out float final )
{
	vec3 OMAX = ( minimum - Origin ) / Direction;
	vec3 OMIN = ( maximum - Origin ) / Direction;
	vec3 MAX = max ( OMAX, OMIN );
	vec3 MIN = min ( OMAX, OMIN );
	final = min ( MAX.x, min ( MAX.y, MAX.z ) );
	start = max ( max ( MIN.x, 0.0), max ( MIN.y, MIN.z ) );	
	return final > start;
}



float GetDepth(vec3 ps)
{
	ps -= pos;

	return clamp(z_far/(z_far-z_near) - (z_far*z_near/(z_far-z_near))/(dot(ps,nav)),0.01,0.99);
}
float rand(vec4 co)
{
	return fract(sin(dot(co,random_seed)) * 1000.0);
}
void main()
{
	float ddd=step_length*global_opacity/2.0;

	vec3 ray = nav;
	vec4 ps = vec4(vertex.xyz-nav*dot(nav,vertex-pos),0.0);

	
	vec4 step  = vec4(ray*step_length,step_length);

	float e=0.0,e0=0.0, start,final;	

	vec2 txt_coord = gl_FragCoord.xy/vec2(width,height);
	vec4 color=texture2D(screen_text, txt_coord);
	vec4 dist_to_polygon = texture2D(distance_map_text, txt_coord).xyzw;



	if(color.w<0.98 && IntersectBox (ps.xyz,ray,box1,box2,start,final ))
	{



		final = min(final,dist_to_polygon.w);
		start = max(start,0.0);
float op0;
float d_alpha;
vec4 cl=vec4(0.0);

		op0 = floor(-e0+start/step_length)+1.0;
		
		ps = ps + step*op0;
		
		ps = ps - step*(rand(vec4(gl_FragCoord.xy,pos.xy)));
		if(ps.w<start)ps+=step;
		e = Equ(ps.xyz-step.xyz);

	for (float itt = 0.0; ps.w<=final && itt < float ( 2000.0 ); itt+=1.0 )
		{

			
				e0 = e;
				
				e = Equ(ps.xyz);
			
			#if use_segmentation_mask
		float i_val = Equ_i(ps.xyz);
		if(i_val==0.0)
		#endif
			{
				
				if(e>min_level)
				{
/*
					cl = clamp((mix(e0,e,1.0/4.0)-min_level)/(max_level-min_level),0.0,1.0);
					d_alpha = (1.0-color.w) * cl.w * ddd;
					color.xyz = color.xyz + d_alpha* cl.xyz;
					color.w = color.w + d_alpha;
*/
					cl = clamp(vec4((mix(e0,e,2.0/4.0)-min_level)/(max_level-min_level)),vec4(0.0),vec4(1.0));
					d_alpha = (1.0-color.w) * cl.w * ddd;
					color.xyz = color.xyz + d_alpha* cl.xyz;
					color.w = color.w + d_alpha;
/*
					cl = clamp((mix(e0,e,3.0/4.0)-min_level)/(max_level-min_level),0.0,1.0);
					d_alpha = (1.0-color.w) * cl.w * ddd;
					color.xyz = color.xyz + d_alpha* cl.xyz;
					color.w = color.w + d_alpha;
*/
					cl = clamp(vec4((e - min_level)/(max_level-min_level)),vec4(0.0),vec4(1.0));
					d_alpha = (1.0-color.w) * cl.w * ddd;
					color.xyz = color.xyz + d_alpha* cl.xyz;
					color.w = color.w + d_alpha;
					
					if(color.w>0.98)	break;
				}
			}
				
			ps += step;
			
		}
		
		if(color.w>0.98)
		{
		
			color.w=1.0;
		}

		if(ps.w>dist_to_polygon.w)
		{
			color.xyz += dist_to_polygon.xyz*(1.0-color.w);
			
			color.w = 1.0;
		}
		
	
	}
	
	
	gl_FragColor = color;
	
	
	
}
